/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.1 Stauverlaufsanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.anastau.anastau;

import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;

/**
 * Klasse zur Verwaltung der globalen Optionen der Stauverlaufsanalyse.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class StauverlaufsAnalyseOptionen {

	/** Aufrufparameter für die Verwendung der Anschlußstellen in einem Stau. */
	private static final String PARAMETER_BENUTZE_ANSCHLUESSE_IM_STAU = "-benutzeAnschluesseImStau";

	/**
	 * Aufrufparameter für die Verwendung der Anschlußstellen vor einem Stau.
	 */
	private static final String PARAMETER_BENUTZE_ANSCHLUESSE_VOR_DEM_STAU = "-benutzeAnschluesseVorDemStau";

	/**
	 * Aufrufparameter für die Verwendung der Anschlußstellen nach einem Stau.
	 */
	private static final String PARAMETER_BENUTZE_ANSCHLUESSE_NACH_DEM_STAU = "-benutzeAnschluesseNachDemStau";

	/** Anschlußstellen im Stau verwenden. */
	private final boolean anschluesseImStau;

	/** Anschlußstellen nach dem Stau verwenden. */
	private final boolean anschluesseNachDemStau;

	/** Anschlußstellen vor dem Stau verwenden. */
	private final boolean anschluesseVorDemStau;

	/**
	 * Konstruktor, nur innerhalb des Packages verfügbar.
	 *
	 * @param argList
	 *            die Argumente, die die Optionen definieren.
	 */
	StauverlaufsAnalyseOptionen(final ArgumentList argList) {
		super();
		if (argList.hasArgument(StauverlaufsAnalyseOptionen.PARAMETER_BENUTZE_ANSCHLUESSE_IM_STAU)) {
			anschluesseImStau = argList.fetchArgument(StauverlaufsAnalyseOptionen.PARAMETER_BENUTZE_ANSCHLUESSE_IM_STAU)
					.booleanValue();
		} else {
			anschluesseImStau = true;
		}

		if (argList.hasArgument(StauverlaufsAnalyseOptionen.PARAMETER_BENUTZE_ANSCHLUESSE_NACH_DEM_STAU)) {
			anschluesseNachDemStau = argList
					.fetchArgument(StauverlaufsAnalyseOptionen.PARAMETER_BENUTZE_ANSCHLUESSE_NACH_DEM_STAU)
					.booleanValue();
		} else {
			anschluesseNachDemStau = true;
		}

		if (argList.hasArgument(StauverlaufsAnalyseOptionen.PARAMETER_BENUTZE_ANSCHLUESSE_VOR_DEM_STAU)) {
			anschluesseVorDemStau = argList
					.fetchArgument(StauverlaufsAnalyseOptionen.PARAMETER_BENUTZE_ANSCHLUESSE_VOR_DEM_STAU)
					.booleanValue();
		} else {
			anschluesseVorDemStau = true;
		}
	}

	/**
	 * liefert Informationen, ob die Messquerschnitte der Zufahrten und
	 * Abfahrten im Stau für die Bestimmung des Zuflusses zum Stau verwendet
	 * werden sollen.
	 *
	 * @return <code>true</code>, wenn die MQ der Zu- und Abfahrten innerhalb
	 *         eines Staus für die Berechnung der zufließenden Verkehrsmenge
	 *         verwendet werden sollen
	 */
	public boolean benutzeAnschluesseImStau() {
		return anschluesseImStau;
	}

	/**
	 * liefert Informationen, ob die Messquerschnitte der Zufahrten und
	 * Abfahrten nach einen Stau für die Bestimmung der Kapazität des Stau
	 * verwendet werden sollen.
	 *
	 * @return <code>true</code>, wenn die MQ der Zu- und Abfahrten nach einem
	 *         Stau für die Berechnung der Kapazität des Stau verwendet werden
	 *         sollen
	 */
	public boolean benutzeAnschluesseNachDemStau() {
		return anschluesseNachDemStau;
	}

	/**
	 * liefert Informationen, ob die Messquerschnitte der Zufahrten und
	 * Abfahrten vor dem Stau für die Bestimmung des Zuflusses zum Stau
	 * verwendet werden sollen.
	 *
	 * @return <code>true</code>, wenn die MQ der Zu- und Abfahrten vor einem
	 *         Stau für die Berechnung der zufließenden Verkehrsmenge verwendet
	 *         werden sollen
	 */
	public boolean benutzeAnschluesseVorDemStau() {
		return anschluesseVorDemStau;
	}
}
