/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.1 Stauverlaufsanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.anastau.progstau;

import java.text.DateFormat;
import java.util.Date;

import com.bitctrl.Constants;
import com.bitctrl.util.Interval;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.ibv.anastau.anastau.SvaMessage;
import de.bsvrz.ibv.anastau.bestimmungstau.stau.IStau;
import de.bsvrz.ibv.anastau.progstau.parameter.ISvpParameterObserver;
import de.bsvrz.ibv.anastau.progstau.parameter.SvpParameterObserver;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.MessQuerschnittAllgemein;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.VerkehrModellNetz;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.onlinedaten.OdVerkehrsDatenKurzZeitMq;
import de.bsvrz.sys.funclib.bitctrl.util.UndefiniertException;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Implementierung des Moduls zur Prognose des Verlaufs eines Stauobjekts.<br>
 * Ein Prognosemodul wird jedem Staubestimmungmodul zugeordnet, d.h. jedem zu
 * verwaltenden VerkehrsmodellNetz.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class StauverlaufsPrognose implements IStauverlaufsPrognose {

	/**
	 * Logger für Debugausgaben.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * liefert den aktuellen QB-Wert des übergebenen Messquerschnitts. Wenn kein
	 * Wert ermittelt werden konnte, wird <code>null</code> geliefert.
	 *
	 * @param mq
	 *            der Messquerschnitt
	 * @return der Wert oder <code>null</code>
	 */
	static Number getMqQbWert(final MessQuerschnittAllgemein mq) {
		Number result = null;

		if (mq != null) {
			final OdVerkehrsDatenKurzZeitMq datenSatz = mq.getOnlineDatensatz(OdVerkehrsDatenKurzZeitMq.class);
			final OdVerkehrsDatenKurzZeitMq.Daten daten = datenSatz
					.abrufenDatum(OdVerkehrsDatenKurzZeitMq.Aspekte.Analyse.getAspekt());

			if (daten.isValid()) {
				result = daten.getWert(OdVerkehrsDatenKurzZeitMq.Daten.Werte.QB.name());
			}
		}

		return result;
	}

	/**
	 * das Modul zur Verwaltung der Parameter der Stauverlaufsprognose.
	 */
	private ISvpParameterObserver parameter;

	@Override
	public final void init(final ClientDavInterface dav, final VerkehrModellNetz netzObjekt) {
		parameter = new SvpParameterObserver(dav, netzObjekt);
	}

	@Override
	public final SvpPrognoseDaten prognose(final IStau stau, final long zeitpunkt) {

		// stau.aktualisierePrognoseVerlaengerung();

		final Interval intervall = new Interval(zeitpunkt, zeitpunkt + parameter.getHorizont());
		final SvpPrognoseDaten result = new SvpPrognoseDaten(zeitpunkt);
		final ISvpZuflussMenge zufluss = new SvpZuflussMenge(stau, intervall, parameter.getDaempfungszeit());
		final ISvpKapazitaet kapazitaet = new SvpKapazitaet(stau, parameter);

		long zs = intervall.getStart();
		double aktuelleLaenge = stau.getStrassenEnde() - stau.getStrassenStart();

		double fzImStau = (aktuelleLaenge * stau.anzahlFahrStreifen((long) aktuelleLaenge)) / parameter.getLaengePkw();

		double maximaleLaenge = aktuelleLaenge;
		long maximaleLaengeZeit = zeitpunkt;

		final long schrittweite = parameter.getSchrittWeite();

		if (schrittweite <= 0) {
			LogTools.log(StauverlaufsPrognose.LOGGER, SvaMessage.ErrorSchrittweitePrognose, schrittweite);
		} else {

			final long stauOffset = stau.getOffset();
			final long laengePkw = parameter.getLaengePkw();
			double lDiff = 0.0;
			double fzDiff = 0.0;
			while (zs <= intervall.getEnd()) {
				try {
					final SvpSchritt schritt = new SvpSchritt();

					final double aktuelleKapazitaet = kapazitaet.getKapazitaet(zs);
					final double aktuellerZufluss = zufluss.getZuflussMenge(zs);

					aktuelleLaenge = Math.max(0.0, aktuelleLaenge + lDiff);
					fzImStau = Math.max(0.0, fzImStau + fzDiff);
					final double verlustZeit = ((fzImStau / aktuelleKapazitaet) * Constants.MILLIS_PER_HOUR);

					schritt.setKapazitaet(aktuelleKapazitaet);
					schritt.setLaenge(aktuelleLaenge);
					if (aktuelleLaenge > maximaleLaenge) {
						maximaleLaenge = aktuelleLaenge;
						maximaleLaengeZeit = zs;
					}
					schritt.setVerlustZeit(Math.round(verlustZeit));
					if (verlustZeit == 0) {
						schritt.setVKfz(0);
					} else {
						schritt.setVKfz(Math.min(254, Math.round(3600.0 * (aktuelleLaenge / verlustZeit))));
					}
					schritt.setZufluss(aktuellerZufluss);

					fzDiff = (aktuellerZufluss - aktuelleKapazitaet)
							* ((double) schrittweite / Constants.MILLIS_PER_HOUR);
					lDiff = (fzDiff / stau.anzahlFahrStreifen((long) (stauOffset + aktuelleLaenge))) * (laengePkw);

					result.add(schritt);
					if (aktuelleLaenge == 0) {
						result.setAufloesungsZeit(zs);
						result.setDauer(zs - intervall.getStart());
						break;
					}
				} catch (final UndefiniertException e) {
					LogTools.log(StauverlaufsPrognose.LOGGER, SvaMessage.WarningKeinePrognoseDaten,
							DateFormat.getDateTimeInstance().format(new Date(zs)), stau, e.getMessage());
				}

				zs += schrittweite;
			}
		}

		result.setSchrittWeite(schrittweite);
		result.setMaxLaenge(Math.max(0, Math.round(maximaleLaenge)));
		result.setMaxLaengeZeit(maximaleLaengeZeit);

		return result;
	}
}
