/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.1 Stauverlaufsanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.anastau.spez;

import java.util.HashSet;
import java.util.Set;

import com.bitctrl.Constants;

import de.bsvrz.ibv.anastau.spez.SpezifikationIT.TestVariante;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.Stau;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.StauListener;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.VerkehrModellNetz;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.onlinedaten.OdStauVerlauf;

/**
 * Klasse zur Auswertung der bei der Ausführung der SWE6.1-Testspezifikation
 * empfangenen Daten der Stauverlaufsanalyse.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class ErgebnisAuswerter implements StauListener {

	/** die Menge der erhaltenen Stauobjekte. */
	private final Set<Stau> stauMenge;

	/** die aktuell ausgeführte Prüfungsphase. */
	private int aktuellePhase = -1;

	/** die verwendete Testvariante. */
	private final TestVariante testVariante;

	/**
	 * Konstruktor.
	 *
	 * @param verkehrModellNetz
	 *            das verwendete VerkehrsModellNetz
	 * @param testVariante
	 *            die verwendete Testvariante
	 */
	public ErgebnisAuswerter(final VerkehrModellNetz verkehrModellNetz, final TestVariante testVariante) {
		this.testVariante = testVariante;
		stauMenge = new HashSet<>();
		verkehrModellNetz.addStauListener(this);
	}

	/**
	 * benachrichtigt den Auswerter über die aktuell laufende Phase des Tests.
	 *
	 * @param phase
	 *            die Nummer der Phase
	 */
	public void phase(final int phase) {
		if (aktuellePhase >= 0) {
			pruefePhase(aktuellePhase);
		}
		aktuellePhase = phase;
	}

	/**
	 * prüft, ob in der aktuellen Prüfungsphase die erwarteten Ergebnisse
	 * empfangen wurden.
	 *
	 * @param phase
	 *            die Nummer der Phase
	 */
	private void pruefePhase(final int phase) {
		for (final Stau stau : stauMenge) {
			final OdStauVerlauf.Daten datum = stau.getOnlineDatensatz(OdStauVerlauf.class)
					.abrufenDatum(OdStauVerlauf.Aspekte.PrognoseNormal.getAspekt());
			if ((datum != null) && datum.isValid()) {
				System.out.println("##############################################");
				System.out.println("# Stauprognose                               #");
				System.out.println("##############################################");
				System.out.println(
						"Staulänge           : " + stau.getSituationsEigenschaften().abrufenDatum().getLaenge());
				System.out.println("Maximale Länge      : " + datum.getMaxLaenge());
				System.out
						.println("Maximale Verlustzeit: " + (datum.getMaxVerlustzeit() / Constants.MILLIS_PER_SECOND));
				System.out.println("Aktuelle Geschw.    : " + datum.getAktuelleGeschwindigkeit());
				System.out.println();
			}
		}

		new PhasenPruefer(phase, testVariante).pruefe(stauMenge);
	}

	@Override
	public void stauAngelegt(final VerkehrModellNetz netz, final Stau stau) {
		stauMenge.add(stau);
	}

	@Override
	public void stauEntfernt(final VerkehrModellNetz netz, final Stau stau) {
		stauMenge.remove(stau);
	}
}
