/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.1 Stauverlaufsanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.anastau.spez;

import java.text.DateFormat;
import java.util.Calendar;
import java.util.Date;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import com.bitctrl.Constants;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.sys.funclib.bitctrl.modell.AnmeldeException;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensendeException;
import de.bsvrz.sys.funclib.bitctrl.modell.ObjektFactory;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.VerkehrModellNetz;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.onlinedaten.OdStoerfallZustand;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.parameter.PdStauBestimmungModell;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.parameter.PdStauPrognoseModell;

/**
 * Die Testfälle der Prüfspezifikation.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class SpezifikationIT {

	/**
	 * Definition der auszuführenden Testvarianten.
	 */
	public enum TestVariante {
		/** Testvariante mit kompletten Durchlauf. */
		MAIN(13),

		/** Parameter verhindert die Stauobjektzusammenfassung in Phase 1. */
		A(3),

		/**
		 * Parameter verhindert die Stauobjektzusammenfassung in Phase 7 und
		 * begrenzt die maximale Stauverlängerung.
		 */
		B(9),

		/**
		 * In Zyklus 8 wird ein Indikator mit einer schlechten Güte geliefert.
		 */
		C(10),

		/** Parameter verhindert die Zerlegung eines Stauobjekts in Phase 9. */
		D(11);

		/**
		 * die Anzahl der Phasen, die in der Variante durchlaufen werden müssen.
		 */
		private final int phasenAnzahl;

		/**
		 * Konstruktor.
		 *
		 * @param anzahl
		 *            die Anzahl der zu durchlaufenden Phasen
		 */
		TestVariante(final int anzahl) {
			this.phasenAnzahl = anzahl;
		}

		/**
		 * liefert die Anzahl der zu druchlaufenden Phasen.
		 *
		 * @return die Anzahl
		 */
		int getPhasenAnzahl() {
			return phasenAnzahl;
		}
	}

	/** das Intervall für die Ausführung der einzelnen Phasen des Tests. */
	private static final long PAUSENZEIT = 3 * Constants.MILLIS_PER_MINUTE;

	/** die verwendete Datenverteilerverbindung. */
	private static ClientDavConnection connection;

	/** der Startzeitpunkt der Testausführung. */
	private long startZeitpunkt;

	/** das verwendete Netz. */
	private TestNetz netz;

	/** die Instanz zur Auswertung der Testergebnisse. */
	private ErgebnisAuswerter ergebnisAuswerter;

	/** die aktuell ausgeführte Testvariante. */
	private TestVariante testVariante = TestVariante.MAIN;

	/** das verwendete Verkehrsmodellnetz. */
	private VerkehrModellNetz verkehrsNetz;

	/**
	 * Ausführung des kompletten Testablaufs nach Variante
	 * {@link TestVariante#MAIN}.
	 *
	 * @throws DatensendeException
	 *             es konnten nicht alle erforderlichen Testdaten an den
	 *             Datenverteiler versendet werden
	 * @throws AnmeldeException
	 *             es konnten nicht alle erforderlichen Anmeldungen beim
	 *             Datenverteiler ausgeführt werden
	 */
	@Test
	public synchronized void ablaufGesamt() throws DatensendeException, AnmeldeException {
		testVariante = TestVariante.MAIN;

		// netz = new TestNetz(startZeitpunkt, PAUSENZEIT, testVariante);
		ergebnisAuswerter = new ErgebnisAuswerter(verkehrsNetz, testVariante);

		setzeParameter(verkehrsNetz, null);
		phasenAusfuehren();
	}

	/**
	 * Ausführung des Testablaufs nach Variante {@link TestVariante#A}.
	 *
	 * @throws DatensendeException
	 *             es konnten nicht alle erforderlichen Testdaten an den
	 *             Datenverteiler versendet werden
	 * @throws AnmeldeException
	 *             es konnten nicht alle erforderlichen Anmeldungen beim
	 *             Datenverteiler ausgeführt werden
	 */
	@Test
	public synchronized void ablaufVarianteA() throws DatensendeException, AnmeldeException {
		testVariante = TestVariante.A;

		// netz = new TestNetz(startZeitpunkt, PAUSENZEIT, testVariante);
		ergebnisAuswerter = new ErgebnisAuswerter(verkehrsNetz, testVariante);

		setzeParameter(verkehrsNetz, null);
		phasenAusfuehren();
	}

	/**
	 * Ausführung des Testablaufs nach Variante {@link TestVariante#B}.
	 *
	 * @throws DatensendeException
	 *             es konnten nicht alle erforderlichen Testdaten an den
	 *             Datenverteiler versendet werden
	 * @throws AnmeldeException
	 *             es konnten nicht alle erforderlichen Anmeldungen beim
	 *             Datenverteiler ausgeführt werden
	 */
	@Test
	public synchronized void ablaufVarianteB() throws DatensendeException, AnmeldeException {
		testVariante = TestVariante.B;

		// netz = new TestNetz(startZeitpunkt, PAUSENZEIT, testVariante);
		ergebnisAuswerter = new ErgebnisAuswerter(verkehrsNetz, testVariante);

		setzeParameter(verkehrsNetz, null);
		phasenAusfuehren();
	}

	/**
	 * Ausführung des Testablaufs nach Variante {@link TestVariante#C}.
	 *
	 * @throws DatensendeException
	 *             es konnten nicht alle erforderlichen Testdaten an den
	 *             Datenverteiler versendet werden
	 * @throws AnmeldeException
	 *             es konnten nicht alle erforderlichen Anmeldungen beim
	 *             Datenverteiler ausgeführt werden
	 */
	@Test
	public synchronized void ablaufVarianteC() throws DatensendeException, AnmeldeException {
		testVariante = TestVariante.C;

		// netz = new TestNetz(startZeitpunkt, PAUSENZEIT, testVariante);
		ergebnisAuswerter = new ErgebnisAuswerter(verkehrsNetz, testVariante);

		setzeParameter(verkehrsNetz, null);
		phasenAusfuehren();
	}

	/**
	 * Ausführung des Testablaufs nach Variante {@link TestVariante#D}.
	 *
	 * @throws DatensendeException
	 *             es konnten nicht alle erforderlichen Testdaten an den
	 *             Datenverteiler versendet werden
	 * @throws AnmeldeException
	 *             es konnten nicht alle erforderlichen Anmeldungen beim
	 *             Datenverteiler ausgeführt werden
	 */
	@Test
	public synchronized void ablaufVarianteD() throws DatensendeException, AnmeldeException {
		testVariante = TestVariante.D;

		// netz = new TestNetz(startZeitpunkt, PAUSENZEIT, testVariante);
		ergebnisAuswerter = new ErgebnisAuswerter(verkehrsNetz, testVariante);

		setzeParameter(verkehrsNetz, null);
		phasenAusfuehren();
	}

	/**
	 * liefert den Startzeitpunkt in Sekunden.
	 *
	 * @return den Zeitpunkt
	 */
	private long berechneStartZeitpunkt() {
		final long jetzt = SpezifikationIT.connection.getTime();
		final Calendar cal = Calendar.getInstance();
		cal.set(Calendar.SECOND, 0);
		cal.set(Calendar.MILLISECOND, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.HOUR_OF_DAY, 0);
		long result = cal.getTimeInMillis();

		while (result < jetzt) {
			result += (2 * SpezifikationIT.PAUSENZEIT);
		}

		if (Math.abs(result - jetzt) < Constants.MILLIS_PER_MINUTE) {
			result += SpezifikationIT.PAUSENZEIT;
		}

		System.out.println("Startzeitpunkt: " + DateFormat.getDateTimeInstance().format(new Date(result)));
		return result;
	}

	/**
	 * stellt die Verbindung zum Testdatenverteiler auf localhost:8083 her.
	 *
	 * @throws AnmeldeException
	 *             erforderliche Anmeldungen beim Datenverteiler konnten nicht
	 *             erfolgreich ausgeführt werden
	 * @throws DatensendeException
	 *             es konnten nicht alle erforderlichen Testdaten über den
	 *             Datenverteiler versendet werden
	 */
	@Before
	public void datenverteilerVerbindung() throws AnmeldeException, DatensendeException {
		try {

			SpezifikationIT.connection = new ClientDavConnection(new ClientDavParameters());
			SpezifikationIT.connection.connect();
			SpezifikationIT.connection.login("Tester", "geheim");

			ObjektFactory.getInstanz().setVerbindung(SpezifikationIT.connection);
			ObjektFactory.getInstanz().registerStandardFactories();

			startZeitpunkt = berechneStartZeitpunkt();

			verkehrsNetz = (VerkehrModellNetz) ObjektFactory.getInstanz().getModellobjekt("netz.swe6Test");
			setzeParameter(verkehrsNetz, null);

			netz = new TestNetz(startZeitpunkt, SpezifikationIT.PAUSENZEIT, testVariante);

			// ergebnisAuswerter = new ErgebnisAuswerter(verkehrsNetz,
			// testVariante);

		} catch (final MissingParameterException e) {
			Assert.fail(e.getMessage());
		} catch (final CommunicationError e) {
			Assert.fail(e.getMessage());
		} catch (final ConnectionException e) {
			Assert.fail(e.getMessage());
		} catch (final InconsistentLoginException e) {
			Assert.fail(e.getMessage());
		}
	}

	/**
	 * der allgemeine Ablauf des Testfalls, der durch die aktuell verwendete
	 * Variante modifiziert wird.
	 *
	 * @throws DatensendeException
	 *             die gewünschten Daten konnten nicht über den Datenverteiler
	 *             versendet werden
	 * @throws AnmeldeException
	 *             es konnten nicht alle erforderlichen Anmeldungen beim
	 *             Datenverteiler ausgeführt werden
	 */
	public synchronized void phasenAusfuehren() throws DatensendeException, AnmeldeException {

		System.out.println("Beginne Testausführung für Variante: " + testVariante);

		startZeitpunkt -= (Constants.MILLIS_PER_MINUTE + SpezifikationIT.PAUSENZEIT);

		for (int phase = 0; phase < testVariante.getPhasenAnzahl(); phase++) {
			setzeParameter(verkehrsNetz, phase);
			netz.phase(phase, testVariante);
			startZeitpunkt += SpezifikationIT.PAUSENZEIT;
			System.out.println("Eingangsdaten für Phase " + phase + " gesetzt: Ergebnisse prüfen um: "
					+ DateFormat.getDateTimeInstance().format(new Date(startZeitpunkt)));
			ergebnisAuswerter.phase(phase);
			SpezifikationIT.connection.sleepUntil(startZeitpunkt);
		}

		startZeitpunkt += (Constants.MILLIS_PER_MINUTE + SpezifikationIT.PAUSENZEIT);
	}

	/**
	 * setzt die für die jeweilige Testvariante vorgesehenen Parameter.
	 *
	 * @param netzObjekt
	 *            das VerkehsmodellNetz, dem die Parameter zugeordnet sind
	 * @param phase
	 *            die aktuelle Phase der testasuführung
	 * @throws AnmeldeException
	 *             es konnten nicht alle erforderlichen Anmeldungen beim
	 *             Datenverteiler ausgeführt werden
	 * @throws DatensendeException
	 *             es konnten nicht alle erforderlichen Daten über den
	 *             Datenverteiler versendet werden
	 */
	private void setzeParameter(final VerkehrModellNetz netzObjekt, final Integer phase)
			throws AnmeldeException, DatensendeException {
		if (phase == null) {
			setzeStauBestimmungsParameter(netzObjekt, phase);
			setzeStauPrognoseParameter(netzObjekt);
		} else {
			switch (testVariante) {
			case B:
				if (phase == 7) {
					setzeStauBestimmungsParameter(netzObjekt, phase);
				}
				break;
			case A:
			case C:
			case D:
			default:
				break;
			}
		}
	}

	/**
	 * setzt die Parameter für die Stauobjektbestimmung.
	 *
	 * @param netzObjekt
	 *            das VerkehsmodellNetz, dem die Parameter zugeordnet sind
	 * @param phase
	 *            die aktuelle Phase der testasuführung
	 * @throws AnmeldeException
	 *             es konnten nicht alle erforderlichen Anmeldungen beim
	 *             Datenverteiler ausgeführt werden
	 * @throws DatensendeException
	 *             es konnten nicht alle erforderlichen Daten über den
	 *             Datenverteiler versendet werden
	 */
	private void setzeStauBestimmungsParameter(final VerkehrModellNetz netzObjekt, final Integer phase)
			throws AnmeldeException, DatensendeException {
		final PdStauBestimmungModell datenSatz = netzObjekt.getParameterDatensatz(PdStauBestimmungModell.class);

		datenSatz.anmeldenSender();
		final PdStauBestimmungModell.Daten datum = datenSatz.erzeugeDatum();
		datum.setZyklusZeit(180);
		datum.setSiVerfahren(OdStoerfallZustand.Aspekte.StoerfallVerfahrenFuzzy.getAspekt());
		datum.setAnfangsLaenge(60);
		datum.setStromAbVerlaengerung(0);
		datum.setMaxSiAbstand(1000);
		datum.setTeilungsLaenge(3000);

		switch (testVariante) {
		case A:
			datum.setMaxSiAbstand(900);
			break;
		case B:
			if ((phase != null) && (phase == 7)) {
				datum.setMaxSiAbstand(750);
			}
			break;
		case D:
			datum.setTeilungsLaenge(6000);
			break;
		default:
			datum.setMaxSiAbstand(1000);
			break;
		}

		datum.setTeilungsverzoegerung(720);
		datum.setPrognoselaengeA1(3000);
		datum.setPrognoselaengeA2(2500);
		datum.setPrognoselaengeA3(5013);
		datum.setMinSIGuete(30);

		datenSatz.sendeDaten(datum);
		datenSatz.abmeldenSender();

	}

	/**
	 * setzt die Parameter für die Stauprognose.
	 *
	 * @param netzObjekt
	 *            das VerkehsmodellNetz, dem die Parameter zugeordnet sind
	 * @throws AnmeldeException
	 *             es konnten nicht alle erforderlichen Anmeldungen beim
	 *             Datenverteiler ausgeführt werden
	 * @throws DatensendeException
	 *             es konnten nicht alle erforderlichen Daten über den
	 *             Datenverteiler versendet werden
	 */
	private void setzeStauPrognoseParameter(final VerkehrModellNetz netzObjekt)
			throws AnmeldeException, DatensendeException {
		final PdStauPrognoseModell datenSatz = netzObjekt.getParameterDatensatz(PdStauPrognoseModell.class);

		datenSatz.anmeldenSender();
		final PdStauPrognoseModell.Daten datum = datenSatz.erzeugeDatum();
		datum.setPrognoseHorizont(360 * Constants.MILLIS_PER_SECOND);
		datum.setAktualisierungsIntervall(2);
		datum.setLaengePkw(6);
		datum.setFaktorQ0(50);
		datum.setDaempfungsZeit(720 * Constants.MILLIS_PER_SECOND);

		datenSatz.sendeDaten(datum);
		datenSatz.abmeldenSender();
	}

	/**
	 * beendet die Datenverteilerverbindung nach Ausführung der Tests.
	 */
	@After
	public void verbindungBeenden() {
		SpezifikationIT.connection.disconnect(false, "Verbindung beendet");
	}
}
