/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.1 Stauverlaufsanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.anastau.spez;

import static de.bsvrz.sys.funclib.bitctrl.modell.verkehr.zustaende.StoerfallSituation.FREIER_VERKEHR;
import static de.bsvrz.sys.funclib.bitctrl.modell.verkehr.zustaende.StoerfallSituation.STAU;
import static de.bsvrz.sys.funclib.bitctrl.modell.verkehr.zustaende.StoerfallSituation.STOERUNG;
import static de.bsvrz.sys.funclib.bitctrl.modell.verkehr.zustaende.StoerfallSituation.ZAEHER_VERKEHR;

import java.util.Map;
import java.util.TreeMap;

import org.junit.Assert;

import com.bitctrl.util.Interval;

import de.bsvrz.iav.gllib.gllib.dav.Messwerte;
import de.bsvrz.ibv.anastau.spez.SpezifikationIT.TestVariante;
import de.bsvrz.sys.funclib.bitctrl.modell.AnmeldeException;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensendeException;
import de.bsvrz.sys.funclib.bitctrl.modell.ObjektFactory;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.MessQuerschnitt;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.MessQuerschnittAllgemein;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.StrassenTeilSegment;

/**
 * das für den Test verwendete Netz.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class TestNetz {

	/** Repräsentation des MQ1 (siehe Prüfspezifikation). */
	private final TestMQ mq1;

	/** Repräsentation des MQ2 (siehe Prüfspezifikation). */
	private final TestMQ mq2;

	/** Repräsentation des MQ3 (siehe Prüfspezifikation). */
	private final TestMQ mq3;

	/** Repräsentation des MQ4 (siehe Prüfspezifikation). */
	private final TestMQ mq4;

	/** Repräsentation des MQ der Abfahrt der AS1 (siehe Prüfspezifikation). */
	private final TestMQ mqAs1Ab;

	/** Repräsentation des MQ der Zufahrt der AS1 (siehe Prüfspezifikation). */
	private final TestMQ mqAs1Zu;

	/** Repräsentation des MQ der Abfahrt der AS2 (siehe Prüfspezifikation). */
	private final TestMQ mqAs2Ab;

	/** Repräsentation des MQ der Zufahrt der AS2 (siehe Prüfspezifikation). */
	private final TestMQ mqAs2Zu;

	/**
	 * Repräsentation des streckenbezogenen Indikators SI1 (siehe
	 * Prüfspezifikation).
	 */
	private final TestSts si1;

	/** das Intervall für die Ausführung der einzelnen Testphasen. */
	private final long intervall;

	/** die aktuelle Phase des Test. */
	private int aktuellePhase;

	/** die aktuelle Testvariante. */
	private TestVariante aktuelleVariante;

	/**
	 * Konstruktor.
	 *
	 * @param startZeitpunkt
	 *            der Startzeitpunkt für die Testausführung
	 * @param intervall
	 *            das Intervall für die Ausführung der einzelnen Testphasen
	 * @param variante
	 *            die verwendete Testvariante
	 * @throws AnmeldeException
	 *             eine erforderliche Anmeldung beim Datenverteiler war nicht
	 *             erfolgreich
	 * @throws DatensendeException
	 *             erforderliche Parameterdaten konnten nicht versendet werden
	 */
	@SuppressWarnings("unused")
	public TestNetz(final long startZeitpunkt, final long intervall, final TestVariante variante)
			throws AnmeldeException, DatensendeException {

		this.intervall = intervall;

		mq1 = new TestMQ1((MessQuerschnitt) ObjektFactory.getInstanz().getModellobjekt("mq.MQ1"), startZeitpunkt,
				intervall);
		mq1.setzePhasen(ZAEHER_VERKEHR, ZAEHER_VERKEHR, ZAEHER_VERKEHR, ZAEHER_VERKEHR, ZAEHER_VERKEHR, ZAEHER_VERKEHR,
				ZAEHER_VERKEHR, ZAEHER_VERKEHR, ZAEHER_VERKEHR, ZAEHER_VERKEHR, ZAEHER_VERKEHR, ZAEHER_VERKEHR,
				ZAEHER_VERKEHR);

		mq2 = new TestMQ2((MessQuerschnitt) ObjektFactory.getInstanz().getModellobjekt("mq.MQ2"), startZeitpunkt,
				intervall);
		mq2.setzePhasen(ZAEHER_VERKEHR, STAU, STAU, STAU, STAU, STAU, STAU, STAU, STAU, STAU, STAU, STAU,
				ZAEHER_VERKEHR);

		mq3 = new TestMQ3((MessQuerschnitt) ObjektFactory.getInstanz().getModellobjekt("mq.MQ3"), startZeitpunkt,
				intervall);

		if (variante == TestVariante.C) {
			mq3.setzePhasen(ZAEHER_VERKEHR, STAU, STAU, STAU, STAU, STAU, STAU, STAU, STAU, FREIER_VERKEHR, STOERUNG,
					FREIER_VERKEHR, FREIER_VERKEHR);
		} else {
			mq3.setzePhasen(ZAEHER_VERKEHR, STAU, STAU, STAU, STAU, STAU, STAU, STAU, STOERUNG, FREIER_VERKEHR,
					STOERUNG, FREIER_VERKEHR, FREIER_VERKEHR);
		}

		mq4 = new TestMQ4((MessQuerschnitt) ObjektFactory.getInstanz().getModellobjekt("mq.MQ4"), startZeitpunkt,
				intervall);
		mq4.setzePhasen(ZAEHER_VERKEHR, ZAEHER_VERKEHR, ZAEHER_VERKEHR, ZAEHER_VERKEHR, ZAEHER_VERKEHR, ZAEHER_VERKEHR,
				ZAEHER_VERKEHR, ZAEHER_VERKEHR, ZAEHER_VERKEHR, ZAEHER_VERKEHR, STOERUNG, FREIER_VERKEHR,
				ZAEHER_VERKEHR);

		mqAs1Ab = new TestMQAs1Ab((MessQuerschnitt) ObjektFactory.getInstanz().getModellobjekt("mq.MQ_AS1Ab"),
				startZeitpunkt, intervall);
		mqAs1Zu = new TestMQAs1Zu((MessQuerschnitt) ObjektFactory.getInstanz().getModellobjekt("mq.MQ_AS1Zu"),
				startZeitpunkt, intervall);
		mqAs2Ab = new TestMQAs2Ab((MessQuerschnitt) ObjektFactory.getInstanz().getModellobjekt("mq.MQ_AS2Ab"),
				startZeitpunkt, intervall);
		mqAs2Zu = new TestMQAs2Zu((MessQuerschnitt) ObjektFactory.getInstanz().getModellobjekt("mq.MQ_AS2Zu"),
				startZeitpunkt, intervall);

		si1 = new TestSts((StrassenTeilSegment) ObjektFactory.getInstanz().getModellobjekt("sts.ass4.sts2"));
		si1.setzePhasen(ZAEHER_VERKEHR, STAU, STAU, STAU, STAU, STAU, STAU, STAU, STAU, STAU, STAU, STOERUNG,
				ZAEHER_VERKEHR);

		new TestSts((StrassenTeilSegment) ObjektFactory.getInstanz().getModellobjekt("sts.ass4.sts3"));
		new GanglinienPrognoseProxy(this);
	}

	/**
	 * liefert die Gangliniendaten für den übergebenen Messquerschnitt und das
	 * angegebenen Intervall.
	 *
	 * @param messQuerschnitt
	 *            der Messquerschnitt
	 * @param prognoseZeitraum
	 *            das Prognoseintervall
	 * @param nurLangfristig
	 *            nur langfristige Prognosedaten liefern
	 * @return die Daten
	 */
	public Map<? extends Long, ? extends Messwerte> getGanglinienDaten(final MessQuerschnittAllgemein messQuerschnitt,
			final Interval prognoseZeitraum, final boolean nurLangfristig) {

		final Map<Long, Messwerte> result = new TreeMap<>();
		final TestMQ mq = TestMQ.getMq(messQuerschnitt);
		if (mq != null) {
			final Boolean anfrageTyp = mq.getGlAnfrageTyp(aktuelleVariante, aktuellePhase);
			if ((anfrageTyp != null) && (anfrageTyp != nurLangfristig)) {
				Assert.fail("Falscher Anfragetyp für Variante in Phase " + aktuellePhase + " an MQ: "
						+ mq.getIndikator() + ", erwartet: " + mq.getGlAnfrageTyp(aktuelleVariante, aktuellePhase)
						+ ", erhalten: " + nurLangfristig);
			}
			long start = prognoseZeitraum.getStart();
			for (final Double wert : mq.getDaten(prognoseZeitraum.getStart(), nurLangfristig)) {
				result.put(start, new Messwerte(wert, 0.0, 50.0, 50.0));
				start += intervall;
			}
		}

		return result;
	}

	/**
	 * führt die angegebene Testphase aus.
	 *
	 * @param phase
	 *            die Nummer der Phase
	 * @param variante
	 *            die verwendete Testvariante
	 * @throws DatensendeException
	 *             die erforderlichen Testdaten konnten nicht erfolgreich
	 *             versendet werden
	 */
	public void phase(final int phase, final TestVariante variante) throws DatensendeException {
		this.aktuellePhase = phase;
		this.aktuelleVariante = variante;
		setzeIndikatoren(phase, variante);
	}

	/**
	 * führt die Operationen der übergebenen Phase aus.
	 *
	 * @param phase
	 *            die Nummer der Phase
	 * @param variante
	 *            die verwendete Testvariante
	 * @throws DatensendeException
	 *             die erforderlichen Daten konnten nicht über den
	 *             Datenverteiler versendet werden
	 */
	private void setzeIndikatoren(final int phase, final TestVariante variante) throws DatensendeException {
		mq1.phase(phase, variante);
		mq2.phase(phase, variante);
		mq3.phase(phase, variante);
		mq4.phase(phase, variante);
		mqAs1Ab.phase(phase, variante);
		mqAs1Zu.phase(phase, variante);
		mqAs2Ab.phase(phase, variante);
		mqAs2Zu.phase(phase, variante);
		si1.phase(phase, variante);
	}
}
